<?php
/**
 * Created by PhpStorm.
 * User: rafaelporto
 * Date: 01/03/2016
 * Time: 10:09
 */

namespace Service\Integration;

use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Zeedhi\Framework\DTO\Request;


class CRMIntegration {

    protected $entityManager;

    protected $CRM_URL;

    public function __construct(EntityManager $entityManager, $crmUrl) {
        $this->entityManager = $entityManager;
        $this->CRM_URL = $crmUrl;
    }

    public function requestCRM($uri, $header = array(), $data = array()){

        $url = $this->CRM_URL.$uri;

        $header[] = 'Expect:';

        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_SAFE_UPLOAD, true);
        curl_setopt($curl_session, CURLOPT_POST, true);
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_TIMEOUT, 86400);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);

        $result = curl_exec($curl_session);
        $error = curl_error($curl_session);

        curl_close($curl_session);

        $result = json_decode($result, true);

        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else if (!empty($result['messages'])) {
            throw new \Exception($result['messages'][0]['message']);
        }  else {
            $result = isset($result['dataset']) ? $result['dataset'] : null;
        }

        return $result;
    }

    public static function requestToken(){
        $integration = \Teknisa\Libs\Util\Utilities::getLoginIntegration();
        $libClientId     = $integration['clientId'];
        $libClientSecret = $integration['clientSecret'];

        $data = array(
            "CLIENT_ID" => $libClientId,
            "CLIENT_SECRET" => $libClientSecret
        );

        $data = json_encode($data);
        $url = Utilities::getTicketUrl().'/lib_requestToken';
        $header = array();
        $header[] = 'Content-Type: application/json';
        $header[] = 'OAuth-KeepConnected: Yes';
        $header[] = 'Content-Length: '.strlen($data);

        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);

        $result = curl_exec($curl_session);
        $error = curl_error($curl_session);

        curl_close($curl_session);

        $result = json_decode($result, true);

        $token = $result["dataset"]["token"][0];

        return array($token,$libClientId);
    }

    public static function requestSetLogin($token,$libClientId){
        $url = Utilities::getTicketUrl().'/setLoggedOperatorInfosInPhpSession';
        $enviroment = Utilities::getEnvironment();
        $data = array(
            'CDOPERADOR'=>$enviroment->getCurrentUserId(),
            'NRORG'=>$enviroment->getCurrentOrganizationId(),
            'NRORGTRAB'=>$enviroment->getCurrentOrganizationId(),
            'CDFILIAL'=>null,
        );
        $data = self::getDataRequestRow($data);
        $data = json_encode($data);
        $header = array();
        $header[] = 'OAuth-Token: '.$token;
        $header[] = 'OAuth-SecretId: '.$libClientId;
        $header[] = 'OAuth-KeepConnected: Yes';
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: '.strlen($data);
        $header[] = 'Expect:';

        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_TIMEOUT, 86400);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($curl_session);
        $error = curl_error($curl_session);
        curl_close($curl_session);
        $result = json_decode($result, true);
        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else if (!empty($result['messages'])) {
            throw new \Exception($result['messages'][0]['message']);
        }  else {
            $result = isset($result['dataset']) ? $result['dataset'] : null;
        }
        return $result;
    }

    public static function requestTicket($uri, $data = array()){

        $url = Utilities::getTicketUrl().$uri;
        list($token,$libClientId) = self::requestToken();
        self::requestSetLogin($token,$libClientId);
        $data = json_encode($data);
        $header = array();
        $header[] = 'OAuth-Token: '.$token;
        $header[] = 'OAuth-SecretId: '.$libClientId;
        $header[] = 'OAuth-KeepConnected: Yes';
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: '.strlen($data);
        $header[] = 'Expect:';

        $curl_session = curl_init();

        curl_setopt($curl_session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl_session, CURLOPT_URL, $url);
        curl_setopt($curl_session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl_session, CURLOPT_TIMEOUT, 86400);
        curl_setopt($curl_session, CURLOPT_RETURNTRANSFER, true);
        $result = curl_exec($curl_session);
        $error = curl_error($curl_session);
        $httpCode = (String)curl_getinfo($curl_session)['http_code'];
        curl_close($curl_session);
        $errorPhp = $result;
        $result = json_decode($result, true);
        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else if (!empty($result['messages'])) {
            throw new \Exception($result['messages'][0]['message']);
        } else if ($httpCode[0] != '2') {
            throw new \Exception($errorPhp);
        }  else {
            $result = isset($result['dataset']) ? $result['dataset'] : null;
        }
        return $result;
    }

    private function createGETRequestURLForAttachment($op, $nrOrg, $cdFilial, $cdOperador, $nrContato, $nrSeqEvolCont, $filename = array()) {
        $getStr = '?OP='.$op;
        $getStr .= '&NRORG='.$nrOrg;
        $getStr .= '&CDFILIAL='.$cdFilial;
        $getStr .= '&CDOPERADOR='.$cdOperador;
        $getStr .= '&NRCONTATO='.$nrContato;
        $getStr .= '&NRSEQEVOLCONT='.$nrSeqEvolCont;
        $getStr .= (count($filename) > 1) ? '&FILENAMES=' : '&FILENAME=';
        $getStr .= implode(',',$filename);

        return $getStr;
    }

    public function addAttachmentsToCRM($nrOrg, $cdFilial, $cdOperador, $nrContato, $nrSeqEvolCont, $file){
        $filename = $file['name'];
        $fileType = $file['type'];
        $filePath = realpath($file['filepath']);

        $getStr = $this->createGETRequestURLForAttachment('CREATE', $nrOrg, $cdFilial, $cdOperador, $nrContato,
            $nrSeqEvolCont, array($filename));

        $cFile = curl_file_create($filePath, $fileType, $filename);
        $postData = array(
            'qqfile' => $cFile
        );

        $header = array();
        $header[] = 'Content-Disposition: multipart/form-data-encoded';

        return $this->requestCRM('/evolContAttachmentsCRUD.php'.$getStr, $header, $postData);
    }

    public function getAttachmentsFromCRM($nrOrg, $cdFilial, $cdOperador, $nrContato, $nrSeqEvolCont) {
        $getStr = $this->createGETRequestURLForAttachment('RETRIEVE', $nrOrg, $cdFilial, $cdOperador, $nrContato, $nrSeqEvolCont);
        return $this->requestCRM('/evolContAttachmentsCRUD.php'.$getStr);
    }

    public function deleteAttachmentFromCRM($nrOrg, $cdFilial, $cdOperador, $nrContato, $nrSeqEvolCont, $filename) {
        $getStr = $this->createGETRequestURLForAttachment('DELETE', $nrOrg, $cdFilial, $cdOperador, $nrContato, $nrSeqEvolCont, array($filename));
        return $this->requestCRM('/evolContAttachmentsCRUD.php'.$getStr);
    }

    private static function getDataRequestDataSet($dataSet) {
        $dataSet = array(
            "requestType" => Request::TYPE_DATA_SET,
            "dataset" => $dataSet
        );

        return $dataSet;
    }

    private static function getDataRequestRow($row) {
        $dataSet = array(
            "requestType" => Request::TYPE_ROW,
            "row" => $row
        );

        return $dataSet;
    }

    public static function addTicketFromCRM($dataSet) {
        $dataSetRequest = self::getDataRequestDataSet($dataSet);

        return self::requestTicket('/getChamadoExterno/save',$dataSetRequest);
    }

    public static function addCommentTicketFromCRM($dataSet) {
        $dataSetRequest = self::getDataRequestDataSet($dataSet);

        return self::requestTicket('/evolcont_contato/save',$dataSetRequest);
    }

    public static function addAttachmentTicketFromCRM($dataSet) {
        $dataSetRequest = self::getDataRequestDataSet($dataSet);

        return self::requestTicket('/crm_anexocontato/save',$dataSetRequest);
    }
}